// Turret.js //

RequireScript("Pulses.js");
RequireScript("Bullets.js");

var Turrets = [];
var TurretSS = LoadSpriteset("Turrets.rss");
var TurretTopSS = LoadSpriteset("TurretTops.rss");

function Turret(x, y)
{
	var turret = Cursor.turretItem;
	this.x = x;
	this.y = y;
	this.a = 0;
	this.type = turret.name;
	this.img = turret.img;
	this.top = TurretTopSS.images[this.img];
	this.bottom = TurretSS.images[this.img];
	this.level = 0;
	this.range = turret.range;
	this.timer = GetTime();
	this.rate = turret.rate;
	this.atk = turret.atk;
	this.price = turret.price;
	this.cost = turret.cost;
	this.active = true;
	this.target = null;
	this.color = Colors.white;
	this.kills = 0;
}

Turret.prototype.blit = function()
{
	var x = MapToScreenX(0, this.x-7);
	var y = MapToScreenY(0, this.y-7);
	this.bottom.blit(x, y);
	this.top.rotateBlitMask(x-8, y-8, this.a, this.color);
}

Turret.prototype.update = function()
{
	if (!this.active) return;
	
	if (this.target == null) this.target = GetClosestEnemy(this.x, this.y, this.range);
	if (this.target == null) return;
	if (Enemies.indexOf(this.target) == -1) { this.target = null; return; }	

	var ex = this.target.x+8;
	var ey = this.target.y+8;

	if (Math.abs(ey-this.y)+Math.abs(ex-this.x) > this.range) { this.target = null; return; }
	
	if (this.type == "Pulser")
	{
		if (this.timer + this.rate < GetTime())
		{
			CreatePulse(this);
			this.target = null;
			this.timer = GetTime();
		}
	}
	else
	{
		this.a = Math.atan2(ey-this.y, ex-this.x);
		if (this.timer + this.rate < GetTime())
		{
			CreateBullet(this);
			this.timer = GetTime();
		}
	}
}

function PlaceTurret(x, y)
{
	var x = Math.floor(x/16)*16+7;
	var y = Math.floor(y/16)*16+7;
	Turrets.push(new Turret(x, y));
}

function GetTurretAt(x, y, exclude)
{
	var i = Turrets.length;
	while(i--)
	{
		var turret = Turrets[i];
		if (exclude) { if (Turrets[i] == Cursor.turret) continue; }
		if ((turret.x)>>4 == x>>4 && (turret.y)>>4 == y>>4) return turret;
	}
	return null;
}

function Upgrade()
{
	var turret = Cursor.turret;
	if (Game.money >= turret.cost) Game.money -= turret.cost;
	else return;
	switch(turret.type)
	{
		case "Gunner":
			turret.atk += 3;
			turret.rate -= 50;
		break;
		case "Laser":
			turret.atk += 2;
		break;
		case "Pulser":
			turret.atk += 2;
			turret.rate -= 40;
			turret.range -= 2;
		break;
		case "Sniper":
			turret.atk += 10;
			turret.range += 1;
			turret.rate -= 500;
		break;
	}
	turret.level++;
	if (turret.level == 1) turret.color = Colors.green;
	if (turret.level == 2) turret.color = Colors.blue;
	if (turret.level == 3) turret.color = Colors.red;
	turret.range += 4;
	turret.price += turret.cost;
	turret.cost *= 2;
	Game.showDropMenu = false;
}

function Sell()
{
	var turret = Cursor.turret;
	Game.money += turret.price;
	Cursor.turret = null;
	Turrets.splice(Turrets.indexOf(turret), 1);
	Game.showDropMenu = false;
}

function MoveTurret()
{
	var turret = Cursor.turret;
	turret.active = false;
	Cursor.move = true;
	Game.showDropMenu = false;
}

function UpdateTurrets()
{
	var i = Turrets.length;
	while(i--) { Turrets[i].update(); }
}

function DrawTurrets()
{
	var i = Turrets.length;
	while(i--) { Turrets[i].blit(); }
}

// Check closest distance...
function GetClosestEnemy(x, y, rad, amt)
{
	var names = [];
	var distances = [];
	var i = Enemies.length;
	while(i--)
	{
		var enemy = Enemies[i];
		var dist = Math.abs(x-(enemy.x+8))+Math.abs(y-(enemy.y+8));
		if (rad) { if ( dist > rad ) continue; };
		distances.push({enemy: enemy, dist: dist});
	}
	distances.sort(SmallestToLargest);
	
	if (distances.length == 0) return null;
	else if (amt) 
	{
		if (distances.length > amt) for (var i = 0; i < amt; ++i) names.push(distances[i].enemy);
		else for (var i = 0; i < distances.length; ++i) names.push(distances[i].enemy)
		return names;
	}
	else return distances[0].enemy;
}

// Magical faries do this:
function SmallestToLargest(a, b)
{
	if (a.dist > b.dist) return 1;
	if (a.dist < b.dist) return -1;
	if (a.dist == b.dist) return 0;
}

// For menu/placement/stat data //
function TurretItem(name, price, atk, range, rate, dir, img, cost)
{
	this.name = name;
	this.range = range;
	this.rate = rate;
	this.atk = atk;
	this.price = price;
	this.dir = dir;
	this.cost = cost;
	this.img = img;
	this.image = LoadSpriteset("Turrets.rss").images[img].createSurface();
	this.image.blitSurface(LoadSpriteset("TurretTops.rss").images[img].createSurface(), -8, -8);
	this.image = this.image.createImage(); 
}

TurretItems = [];
TurretItems[0] = new TurretItem("Gunner", 10, 4, 40, 400, "Gunner", 0, 8);
TurretItems[1] = new TurretItem("Pulser", 15, 3, 32, 500, "Pulser", 2, 12);
TurretItems[2] = new TurretItem("Laser", 20, 2, 32, 100, "Laser", 1, 10);
TurretItems[3] = new TurretItem("Sniper", 35, 40, 80, 2500, "Sniper", 3, 15);